package org.fai.fs;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;

import javax.swing.JFrame;

import de.erichseifert.gral.data.DataSeries;
import de.erichseifert.gral.data.DataTable;
import de.erichseifert.gral.plots.BarPlot;
import de.erichseifert.gral.plots.PlotArea;
import de.erichseifert.gral.plots.XYPlot;
import de.erichseifert.gral.plots.axes.AxisRenderer;
import de.erichseifert.gral.plots.points.DefaultPointRenderer2D;
import de.erichseifert.gral.plots.points.PointRenderer;
import de.erichseifert.gral.plots.lines.DefaultLineRenderer2D;
import de.erichseifert.gral.plots.lines.LineRenderer;
import de.erichseifert.gral.ui.InteractivePanel;
import de.erichseifert.gral.util.Insets2D;

public class MyPlot extends JFrame {
    public MyPlot(String title, double[] x, double[] y, double[] z) {
        // Create data
        DataTable data = new DataTable(Double.class, Double.class, Double.class);

        int POINT_COUNT = x.length;
        for (int i = 0; i < POINT_COUNT; i++) {
            data.add(x[i], y[i], z[i]);
        }

        // Create series
        DataSeries series1 = new DataSeries("Series 1", data, 0, 1);
        DataSeries series2 = new DataSeries("Series 2", data, 0, 2);
        XYPlot plot = new XYPlot(series1, series2);

        // Style the plot
        double insetsTop = 20.0,
               insetsLeft = 60.0,
               insetsBottom = 60.0,
               insetsRight = 40.0;
        plot.setInsets(new Insets2D.Double(
                insetsTop, insetsLeft, insetsBottom, insetsRight));
        plot.setSetting(BarPlot.TITLE, title);

        // Style the plot area
        plot.getPlotArea().setSetting(
                PlotArea.COLOR, new Color(0.0f, 0.3f, 1.0f));
        plot.getPlotArea().setSetting(
                PlotArea.BORDER, new BasicStroke(2f));

        // Style data series
        PointRenderer points1 = new DefaultPointRenderer2D();
        points1.setSetting(PointRenderer.SHAPE, new Ellipse2D.Double(-3.0, -3.0, 6.0, 6.0));
        points1.setSetting(PointRenderer.COLOR, new Color(1.0f, 0.0f, 0.0f, 0.3f));
        plot.setPointRenderer(series1, points1);

		LineRenderer line = new DefaultLineRenderer2D();
        line.setSetting(LineRenderer.COLOR, new Color(0.0f, 1.0f, 0.0f, 0.3f));
        plot.setLineRenderer(series2, line);

        // Style axes
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.LABEL, "X");
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.LABEL, "Y");
        //plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.TICKS_SPACING, 1.0);
        //plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.TICKS_SPACING, 2.0);
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);

        // Display on screen
        getContentPane().add(new InteractivePanel(plot), BorderLayout.CENTER);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setMinimumSize(getContentPane().getMinimumSize());
        setSize(504, 327);
    }

    public MyPlot(String title, double[] x, double[] y) {
        // Create data
        DataTable data = new DataTable(Double.class, Double.class);

        int POINT_COUNT = x.length;
        for (int i = 0; i < POINT_COUNT; i++) {
            data.add(x[i], y[i]);
        }

        // Create series
        DataSeries series1 = new DataSeries("Series 1", data, 0, 1);
        XYPlot plot = new XYPlot(series1);

        // Style the plot
        double insetsTop = 20.0,
               insetsLeft = 60.0,
               insetsBottom = 60.0,
               insetsRight = 40.0;
        plot.setInsets(new Insets2D.Double(
                insetsTop, insetsLeft, insetsBottom, insetsRight));
        plot.setSetting(BarPlot.TITLE, title);

        // Style the plot area
        plot.getPlotArea().setSetting(
                PlotArea.COLOR, new Color(0.0f, 0.3f, 1.0f));
        plot.getPlotArea().setSetting(
                PlotArea.BORDER, new BasicStroke(2f));

        // Style data series
        PointRenderer points1 = new DefaultPointRenderer2D();
        points1.setSetting(PointRenderer.SHAPE, new Ellipse2D.Double(-3.0, -3.0, 6.0, 6.0));
        points1.setSetting(PointRenderer.COLOR, new Color(0.0f, 0.0f, 0.5f, 0.3f));
        plot.setPointRenderer(series1, points1);


        // Style axes
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.LABEL, "X");
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.LABEL, "Y");
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.TICKS_SPACING, 0.01);
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.TICKS_SPACING, 2.0);
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);

        // Display on screen
        getContentPane().add(new InteractivePanel(plot), BorderLayout.CENTER);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setMinimumSize(getContentPane().getMinimumSize());
        setSize(504, 327);
    }

    public MyPlot(String title, int type, String yTitle, double[] x1, double[] y1, double[] x2, double[] y2) {
        // Create data
        DataTable data1 = new DataTable(Double.class, Double.class);
        DataTable data2 = new DataTable(Double.class, Double.class);

        for (int i = 0; i < x1.length; i++) data1.add(x1[i], y1[i]);
        for (int i = 0; i < x2.length; i++) data2.add(x2[i], y2[i]);

        // Create series
        DataSeries series1 = new DataSeries("Series 1", data1, 0, 1);
        DataSeries series2 = new DataSeries("Series 2", data2, 0, 1);
        XYPlot plot = new XYPlot(series1, series2);

        // Style the plot
        double insetsTop = 20.0,
               insetsLeft = 60.0,
               insetsBottom = 60.0,
               insetsRight = 40.0;
        plot.setInsets(new Insets2D.Double(
                insetsTop, insetsLeft, insetsBottom, insetsRight));
        plot.setSetting(BarPlot.TITLE, title);

        // Style the plot area
        plot.getPlotArea().setSetting(
                PlotArea.COLOR, new Color(0.0f, 0.3f, 1.0f));
        plot.getPlotArea().setSetting(
                PlotArea.BORDER, new BasicStroke(2f));

        // Style data series
        PointRenderer points1 = new DefaultPointRenderer2D();
        points1.setSetting(PointRenderer.SHAPE, new Ellipse2D.Double(-3.0, -3.0, 6.0, 6.0));
        points1.setSetting(PointRenderer.COLOR, new Color(1.0f, 0.0f, 0.0f, 0.3f));
        plot.setPointRenderer(series1, points1);

		if (type == 1) {
			PointRenderer points2 = new DefaultPointRenderer2D();
			points2.setSetting(PointRenderer.SHAPE, new Ellipse2D.Double(-3.0, -3.0, 6.0, 6.0));
			points2.setSetting(PointRenderer.COLOR, new Color(0.0f, 0.0f, 1.0f, 0.3f));
			plot.setPointRenderer(series2, points2);
		} else if (type == 2) {
			plot.setPointRenderer(series2, null);
			LineRenderer line = new DefaultLineRenderer2D();
			line.setSetting(LineRenderer.COLOR, new Color(0.0f, 0.0f, 0.0f, 0.8f));
			plot.setLineRenderer(series2, line);
		}
        // Style axes
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.LABEL, (type == 1 ? "Pressure [hPa]" : "Time / MaxTime"));
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.LABEL, yTitle);
        //plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.TICKS_SPACING, 1.0);
        //plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.TICKS_SPACING, 2.0);
        plot.getAxisRenderer(XYPlot.AXIS_X).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);
        plot.getAxisRenderer(XYPlot.AXIS_Y).setSetting(AxisRenderer.INTERSECTION, -Double.MAX_VALUE);

        // Display on screen
        getContentPane().add(new InteractivePanel(plot), BorderLayout.CENTER);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setMinimumSize(getContentPane().getMinimumSize());
        setSize(800, 600);
    }
	
}
